﻿using EPiServer.Personalization.VisitorGroups;
using EPiServer.Security;
using EPiServer.ServiceLocation;
using Mediachase.Commerce;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Catalog.Objects;
using Mediachase.Commerce.Marketing;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Pricing;
using Mediachase.MetaDataPlus.Configurator;
using System;
using System.Collections.Generic;
using System.Security.Principal;

namespace EPiServer.Commerce.Addons.Promotions
{ 
    public class CustomPromotionEntryPopulate : IPromotionEntryPopulate
    {
        // Methods
        public void Populate(PromotionEntry entry, LineItem lineItem)
        {
            entry.Quantity = lineItem.Quantity;
            entry.Owner = lineItem;
            entry["LineItemId"] = lineItem.LineItemId;
            entry["ShippingAddressId"] = lineItem.ShippingAddressId;
            entry["ExtendedPrice"] = lineItem.ExtendedPrice;
            entry["MinQuantity"] = lineItem.MinQuantity;
            entry["MaxQuantity"] = lineItem.MaxQuantity;
            entry["LineItemDiscountAmount"] = lineItem.LineItemDiscountAmount;
            entry["OrderLevelDiscountAmount"] = lineItem.OrderLevelDiscountAmount;
            entry["ShippingMethodName"] = lineItem.ShippingMethodName ?? string.Empty;
            entry["ExtendedPrice"] = lineItem.ExtendedPrice;
            entry["Description"] = lineItem.Description ?? string.Empty;
            entry["Status"] = lineItem.Status ?? string.Empty;
            entry["DisplayName"] = lineItem.DisplayName ?? string.Empty;
            entry["AllowBackordersAndPreorders"] = lineItem.AllowBackordersAndPreorders;
            entry["InStockQuantity"] = lineItem.InStockQuantity;
            entry["PreorderQuantity"] = lineItem.PreorderQuantity;
            entry["BackorderQuantity"] = lineItem.BackorderQuantity;
            entry["InventoryStatus"] = lineItem.InventoryStatus;
            foreach (MetaField field in lineItem.MetaClass.MetaFields)
            {
                entry[field.Name] = lineItem[field];
            }

            PopulateVisitorGroupEntry(ref entry);
        }

        public void PopulateVisitorGroupEntry(ref PromotionEntry entry)
        {
            // KLUDGE: For demo purposes only. Retrieve the last time we made this check and the principal user at the 
            // time from the cache.
            IPrincipal lastPrincipal = null;
            double lastTime = 0;
            try
            {
                lastPrincipal = CacheManager.Get("promoLastPrincipal") as IPrincipal;
                lastTime = (double)CacheManager.Get("promoLastTime");
            }
            catch (NullReferenceException)
            {
                // Catch any nulls on the double case above. Then continue...
            }            

            // KLUDGE: For demo purposes only. For performance, only re-check VG's if more than 5 seconds have passed or
            // if the principal has changed.
            if (((DateTime.Now.TimeOfDay.TotalSeconds - lastTime) >= 5) || (!PrincipalInfo.CurrentPrincipal.Equals(lastPrincipal)))            
            {
                // Get all the VG repository and all roles available within it
                VisitorGroupRoleRepository vgRepository = VisitorGroupRole.GetRepository();
                IEnumerable<string> vgRoles = vgRepository.GetAllRoles();
                VisitorGroupHelper vgHelper = new VisitorGroupHelper();
                string tempVisitorRoles = "";
            
                // Loop through all VG's to see if the current visitor is a member of any
                foreach(string role in vgRoles)
                {
                    // Is the visitor in the role?
                    if (vgHelper.IsPrincipalInGroup(PrincipalInfo.CurrentPrincipal, role))
                    {
                        // If there is a match add the role to the temp string with a comma delimiter. This will allow for creating of
                        // an expression in the Commerce Manager Promotions area for any number of visitor groups. For example, you 
                        // can specify if the Visitor Group "Contains" the "US Visitor" VG, give 10% off.
                        tempVisitorRoles = tempVisitorRoles + role + ",";
                    }
                }

                // Update the entry with all roles the visitor is a member of
                entry["VisitorGroup"] = tempVisitorRoles;
            }

            // KLUDGE: For demo purposes only. Cache the last time we made this check and the principal user at the time.
            CacheManager.Add("promoLastTime", DateTime.Now.TimeOfDay.TotalSeconds);
            CacheManager.Add("promoLastPrincipal", PrincipalInfo.CurrentPrincipal);         
        }

        public void Populate(ref PromotionEntry promotionEntry, object val)
        {
            throw new NotSupportedException("This method is obsolete.");
        }

        public void Populate(PromotionEntry entry, Entry catalogEntry, MarketId marketId, Currency currency)
        {
            entry.Quantity = 1M;
            entry.Owner = catalogEntry;
            entry["Id"] = catalogEntry.ID;
            if (catalogEntry.ItemAttributes != null)
            {
                entry["MinQuantity"] = catalogEntry.ItemAttributes.MinQuantity;
                entry["MaxQuantity"] = catalogEntry.ItemAttributes.MaxQuantity;
                if (catalogEntry.ItemAttributes.Attribute != null)
                {
                    foreach (ItemAttribute attribute in catalogEntry.ItemAttributes.Attribute)
                    {
                        if ((attribute.Value != null) && (attribute.Value.Length > 0))
                        {
                            entry[attribute.Name] = attribute.Value[0];
                        }
                    }
                }
            }
            if (catalogEntry.Inventory != null)
            {
                entry["AllowBackordersAndPreorders"] = !catalogEntry.Inventory.AllowBackorder ? ((object) 0) : ((object) catalogEntry.Inventory.AllowPreorder);
                entry["InStockQuantity"] = catalogEntry.Inventory.InStockQuantity;
                entry["PreorderQuantity"] = catalogEntry.Inventory.PreorderQuantity;
                entry["BackorderQuantity"] = catalogEntry.Inventory.BackorderQuantity;
                entry["InventoryStatus"] = catalogEntry.Inventory.InventoryStatus;
            }
            IPriceValue value2 = ServiceLocator.Current.GetInstance<IPriceService>().GetDefaultPrice(marketId, FrameworkContext.Current.CurrentDateTime, new CatalogKey(catalogEntry), currency);
            if (value2 != null)
            {
                entry["ExtendedPrice"] = value2.UnitPrice.Amount;
            }

            PopulateVisitorGroupEntry(ref entry);
        }

        public void PopulateCustom(PromotionEntry promotionEntry, object obj)
        {
            throw new NotSupportedException();
        }
    }
}
